@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -122,7 +122,7 @@ class PlotWindow(PlotWidget):
         self._curvesROIDockWidget = None
         self._maskToolsDockWidget = None
         self._consoleDockWidget = None
-        self._statsWidget = None
+        self._statsDockWidget = None
 
         # Create color bar, hidden by default for backward compatibility
         self._colorbar = ColorBarWidget(parent=self, plot=self)
@@ -224,58 +224,34 @@ class PlotWindow(PlotWidget):
         self._sigAxesVisibilityChanged.connect(self._updateColorBarBackground)
         self._updateColorBarBackground()
 
-        gridLayout = qt.QGridLayout()
-        gridLayout.setSpacing(0)
-        gridLayout.setContentsMargins(0, 0, 0, 0)
-        gridLayout.addWidget(self.getWidgetHandle(), 0, 0)
-        gridLayout.addWidget(self._colorbar, 0, 1)
-        gridLayout.setRowStretch(0, 1)
-        gridLayout.setColumnStretch(0, 1)
-        centralWidget = qt.QWidget(self)
-        centralWidget.setLayout(gridLayout)
-        self.setCentralWidget(centralWidget)
+        if control:  # Create control button only if requested
+            self.controlButton = qt.QToolButton()
+            self.controlButton.setText("Options")
+            self.controlButton.setToolButtonStyle(qt.Qt.ToolButtonTextBesideIcon)
+            self.controlButton.setAutoRaise(True)
+            self.controlButton.setPopupMode(qt.QToolButton.InstantPopup)
+            menu = qt.QMenu(self)
+            menu.aboutToShow.connect(self._customControlButtonMenu)
+            self.controlButton.setMenu(menu)
 
         self._positionWidget = None
+        if position:  # Add PositionInfo widget to the bottom of the plot
+            if isinstance(position, abc.Iterable):
+                # Use position as a set of converters
+                converters = position
+            else:
+                converters = None
+            self._positionWidget = tools.PositionInfo(
+                plot=self, converters=converters)
+            # Set a snapping mode that is consistent with legacy one
+            self._positionWidget.setSnappingMode(
+                tools.PositionInfo.SNAPPING_CROSSHAIR |
+                tools.PositionInfo.SNAPPING_ACTIVE_ONLY |
+                tools.PositionInfo.SNAPPING_SYMBOLS_ONLY |
+                tools.PositionInfo.SNAPPING_CURVE |
+                tools.PositionInfo.SNAPPING_SCATTER)
 
-        if control or position:
-            hbox = qt.QHBoxLayout()
-            hbox.setContentsMargins(0, 0, 0, 0)
-
-            if control:
-                self.controlButton = qt.QToolButton()
-                self.controlButton.setText("Options")
-                self.controlButton.setToolButtonStyle(qt.Qt.ToolButtonTextBesideIcon)
-                self.controlButton.setAutoRaise(True)
-                self.controlButton.setPopupMode(qt.QToolButton.InstantPopup)
-                menu = qt.QMenu(self)
-                menu.aboutToShow.connect(self._customControlButtonMenu)
-                self.controlButton.setMenu(menu)
-
-                hbox.addWidget(self.controlButton)
-
-            if position:  # Add PositionInfo widget to the bottom of the plot
-                if isinstance(position, abc.Iterable):
-                    # Use position as a set of converters
-                    converters = position
-                else:
-                    converters = None
-                self._positionWidget = tools.PositionInfo(
-                    plot=self, converters=converters)
-                # Set a snapping mode that is consistent with legacy one
-                self._positionWidget.setSnappingMode(
-                    tools.PositionInfo.SNAPPING_CROSSHAIR |
-                    tools.PositionInfo.SNAPPING_ACTIVE_ONLY |
-                    tools.PositionInfo.SNAPPING_SYMBOLS_ONLY |
-                    tools.PositionInfo.SNAPPING_CURVE |
-                    tools.PositionInfo.SNAPPING_SCATTER)
-
-                hbox.addWidget(self._positionWidget)
-
-            hbox.addStretch(1)
-            bottomBar = qt.QWidget(centralWidget)
-            bottomBar.setLayout(hbox)
-
-            gridLayout.addWidget(bottomBar, 1, 0, 1, -1)
+        self.__setCentralWidget()
 
         # Creating the toolbar also create actions for toolbuttons
         self._interactiveModeToolBar = tools.InteractiveModeToolBar(
@@ -296,6 +272,41 @@ class PlotWindow(PlotWidget):
             for action in toolbar.actions():
                 self.addAction(action)
 
+    def __setCentralWidget(self):
+        """Set central widget to host plot backend, colorbar, and bottom bar"""
+        gridLayout = qt.QGridLayout()
+        gridLayout.setSpacing(0)
+        gridLayout.setContentsMargins(0, 0, 0, 0)
+        gridLayout.addWidget(self.getWidgetHandle(), 0, 0)
+        gridLayout.addWidget(self._colorbar, 0, 1)
+        gridLayout.setRowStretch(0, 1)
+        gridLayout.setColumnStretch(0, 1)
+        centralWidget = qt.QWidget(self)
+        centralWidget.setLayout(gridLayout)
+
+        if hasattr(self, "controlButton") or self._positionWidget is not None:
+            hbox = qt.QHBoxLayout()
+            hbox.setContentsMargins(0, 0, 0, 0)
+
+            if hasattr(self, "controlButton"):
+                hbox.addWidget(self.controlButton)
+
+            if self._positionWidget is not None:
+                hbox.addWidget(self._positionWidget)
+
+            hbox.addStretch(1)
+            bottomBar = qt.QWidget(centralWidget)
+            bottomBar.setLayout(hbox)
+
+            gridLayout.addWidget(bottomBar, 1, 0, 1, -1)
+
+        self.setCentralWidget(centralWidget)
+
+    @docstring(PlotWidget)
+    def setBackend(self, backend):
+        super(PlotWindow, self).setBackend(backend)
+        self.__setCentralWidget()  # Recreate PlotWindow's central widget
+
     @docstring(PlotWidget)
     def setBackgroundColor(self, color):
         super(PlotWindow, self).setBackgroundColor(color)
@@ -313,7 +324,7 @@ class PlotWindow(PlotWidget):
 
     def _updateColorBarBackground(self):
         """Update the colorbar background according to the state of the plot"""
-        if self._isAxesDisplayed():
+        if self.isAxesDisplayed():
             color = self.getBackgroundColor()
         else:
             color = self.getDataBackgroundColor()
@@ -464,7 +475,7 @@ class PlotWindow(PlotWidget):
         """Add a dock widget as a new tab if there are already dock widgets
         in the plot. When the first tab is added, the area is chosen
         depending on the plot geometry:
-        it the window is much wider than it is high, the right dock area
+        if the window is much wider than it is high, the right dock area
         is used, else the bottom dock area is used.
 
         :param dock_widget: Instance of :class:`QDockWidget` to be added.
@@ -485,6 +496,33 @@ class PlotWindow(PlotWidget):
             self.tabifyDockWidget(self._dockWidgets[0],
                                   dock_widget)
 
+    def removeDockWidget(self, dockwidget):
+        """Removes the *dockwidget* from the main window layout and hides it.
+
+        Note that the *dockwidget* is *not* deleted.
+
+        :param QDockWidget dockwidget:
+        """
+        if dockwidget in self._dockWidgets:
+            self._dockWidgets.remove(dockwidget)
+        super(PlotWindow, self).removeDockWidget(dockwidget)
+
+    def __handleFirstDockWidgetShow(self, visible):
+        """Handle QDockWidget.visibilityChanged
+
+        It calls :meth:`addTabbedDockWidget` for the `sender` widget.
+        This allows to call `addTabbedDockWidget` lazily.
+
+        It disconnect itself from the signal once done.
+
+        :param bool visible:
+        """
+        if visible:
+            dockWidget = self.sender()
+            dockWidget.visibilityChanged.disconnect(
+                self.__handleFirstDockWidgetShow)
+            self.addTabbedDockWidget(dockWidget)
+
     def getColorBarWidget(self):
         """Returns the embedded :class:`ColorBarWidget` widget.
 
@@ -499,7 +537,8 @@ class PlotWindow(PlotWidget):
         if self._legendsDockWidget is None:
             self._legendsDockWidget = LegendsDockWidget(plot=self)
             self._legendsDockWidget.hide()
-            self.addTabbedDockWidget(self._legendsDockWidget)
+            self._legendsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._legendsDockWidget
 
     def getCurvesRoiDockWidget(self):
@@ -509,7 +548,8 @@ class PlotWindow(PlotWidget):
             self._curvesROIDockWidget = CurvesROIDockWidget(
                 plot=self, name='Regions Of Interest')
             self._curvesROIDockWidget.hide()
-            self.addTabbedDockWidget(self._curvesROIDockWidget)
+            self._curvesROIDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._curvesROIDockWidget
 
     def getCurvesRoiWidget(self):
@@ -529,7 +569,8 @@ class PlotWindow(PlotWidget):
             self._maskToolsDockWidget = MaskToolsDockWidget(
                 plot=self, name='Mask')
             self._maskToolsDockWidget.hide()
-            self.addTabbedDockWidget(self._maskToolsDockWidget)
+            self._maskToolsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._maskToolsDockWidget
 
     def getStatsWidget(self):
@@ -537,16 +578,18 @@ class PlotWindow(PlotWidget):
 
         :rtype: BasicStatsWidget
         """
-        if self._statsWidget is None:
-            dockWidget = qt.QDockWidget(parent=self)
-            dockWidget.setWindowTitle("Curves stats")
-            dockWidget.layout().setContentsMargins(0, 0, 0, 0)
-            self._statsWidget = BasicStatsWidget(parent=self, plot=self)
-            self._statsWidget.sigVisibilityChanged.connect(self.getStatsAction().setChecked)
-            dockWidget.setWidget(self._statsWidget)
-            dockWidget.hide()
-            self.addTabbedDockWidget(dockWidget)
-        return self._statsWidget
+        if self._statsDockWidget is None:
+            self._statsDockWidget = qt.QDockWidget()
+            self._statsDockWidget.setWindowTitle("Curves stats")
+            self._statsDockWidget.layout().setContentsMargins(0, 0, 0, 0)
+            statsWidget = BasicStatsWidget(parent=self, plot=self)
+            self._statsDockWidget.setWidget(statsWidget)
+            statsWidget.sigVisibilityChanged.connect(
+                self.getStatsAction().setChecked)
+            self._statsDockWidget.hide()
+            self._statsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
+        return self._statsDockWidget.widget()
 
     # getters for actions
     @property
@@ -807,6 +850,9 @@ class Plot1D(PlotWindow):
             self.setWindowTitle('Plot1D')
         self.getXAxis().setLabel('X')
         self.getYAxis().setLabel('Y')
+        action = self.getFitAction()
+        action.setXRangeUpdatedOnZoom(True)
+        action.setFittedItemUpdatedFromActiveCurve(True)
 
 
 class Plot2D(PlotWindow):
@@ -895,37 +941,30 @@ class Plot2D(PlotWindow):
         :param float y: Y position in plot coordinates
         :return: The value at that point or '-'
         """
-        value = '-'
-        valueZ = -float('inf')
-        mask = 0
-        maskZ = -float('inf')
-
-        for image in self.getAllImages():
-            data = image.getData(copy=False)
-            isMask = isinstance(image, items.MaskImageData)
-            if isMask:
-                zIndex = maskZ
+        pickedMask = None
+        for picked in self.pickItems(
+                *self.dataToPixel(x, y, check=False),
+                lambda item: isinstance(item, items.ImageBase)):
+            if isinstance(picked.getItem(), items.MaskImageData):
+                if pickedMask is None:  # Use top-most if many masks
+                    pickedMask = picked
             else:
-                zIndex = valueZ
-            if image.getZValue() >= zIndex:
-                # This image is over the previous one
-                ox, oy = image.getOrigin()
-                sx, sy = image.getScale()
-                row, col = (y - oy) / sy, (x - ox) / sx
-                if row >= 0 and col >= 0:
-                    # Test positive before cast otherwise issue with int(-0.5) = 0
-                    row, col = int(row), int(col)
-                    if (row < data.shape[0] and col < data.shape[1]):
-                        v, z = data[row, col], image.getZValue()
-                        if not isMask:
-                            value = v
-                            valueZ = z
-                        else:
-                            mask = v
-                            maskZ = z
-        if maskZ > valueZ and mask > 0:
-            return value, "Masked"
-        return value
+                image = picked.getItem()
+
+                indices = picked.getIndices(copy=False)
+                if indices is not None:
+                    row, col = indices[0][0], indices[1][0]
+                    value = image.getData(copy=False)[row, col]
+
+                    if pickedMask is not None:  # Check if masked
+                        maskItem = pickedMask.getItem()
+                        indices = pickedMask.getIndices()
+                        row, col = indices[0][0], indices[1][0]
+                        if maskItem.getData(copy=False)[row, col] != 0:
+                            return value, "Masked"
+                    return value
+
+        return '-'  # No image picked
 
     def _getImageDims(self, *args):
         activeImage = self.getActiveImage()
